angular.module('WebTrax.services').service('WebTraxServices',

function WebTraxServices($http, $q, $timeout, ServiceConfig, UserInfo) {
	
	var DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss.S [CDT]';

	var self = this;

	self.cachedWgList = null;
	self.cachedWrList = null;
	self.cachedWgId = '';


	self.getWgList = function (opts) {
		opts = opts || {cache: true};
		var def = $q.defer();
		
		//if cached result is more than 1 minutes old, we will still make server call
		if(opts.cache && self.cachedWgList && (((new Date()).getTime() - self.cachedWgList.lastUpdated.getTime()) < 1*60*1000)) {
			def.resolve(self.cachedWgList);
			return def.promise;
		}
		
		$http.get(ServiceConfig.ServicesURL + '/workgroups?date=' + new Date().getTime())
		.success (function (data, status, headers, config, statusText) {
			if (_isErrorResponse (data)) {
				self.cachedWgList = null;
				def.reject(data.statusMessage);
				return;
			}

			try {
				UserInfo.firstName = data.userFirstName;
				UserInfo.lastName = data.userLastName;
				
				var groupedWgList = [];
				
				var wgList = data.grpList;
				
				//Group workgroups by groupType 
				while (wgList.length > 0) {
					var wg = wgList.shift();
					
					var i = 0;
					for (; i < groupedWgList.length; ++i) {
						if (wg.groupType === groupedWgList[i].groupType) {
							break;
						}
					}
					
					//Get the list of workgroups for the groupType or create a new one 
					var group;
					if (i == groupedWgList.length) {
						//groupType is not in the list yet, push a new one
						group = {groupType: wg.groupType, wgList: []};
						groupedWgList.push(group);
					} else {
						group = groupedWgList[i];
					}
					
					//Push workgroup to the end of grouped list
					group.wgList.push(wg);
				}
				self.cachedWgList = groupedWgList;
				self.cachedWgList.lastUpdated = new Date();
		
				def.resolve(groupedWgList);
			} catch (err) {
				console.error(err);
				def.reject('Failed to load data: internal error');
			}
		})
		.error (function (data, status, headers, config, statusText) {
			self.cachedWgList = null;
			def.reject('Failed to load data: ' + statusText);
		});
		
		return def.promise;
	}

	self.getWrList = function (wgId, opts) {
		opts = opts || {cache: true};
		var def = $q.defer();
		
		//if cached result is more than 1 minutes old, we will still make server call
		if(opts.cache && self.cachedWgId == wgId && self.cachedWrList && (((new Date()).getTime() - self.cachedWrList.lastUpdated.getTime()) < 1*60*1000)) {
			def.resolve(self.cachedWrList);
			return def.promise;
		}
		
		$http.get(ServiceConfig.ServicesURL + '/workgroups/' + wgId + '/workrequests?date=' + new Date().getTime())
		.success (function (data, status, headers, config, statusText) {
			if (_isErrorResponse (data)) {
				self.cachedWrList = null;
				self.cachedWgId = '';
				def.reject(data.statusMessage);
				return;
			}
			
			try {
				angular.forEach(data.wrList, function (wr) {
					wr.modifiedDate = new Date(wr.lastModifiedTime);
				});
		
				self.cachedWgId = wgId;
				self.cachedWrList = data;
				self.cachedWrList.lastUpdated = new Date();
				def.resolve(self.cachedWrList);
			} catch (err) {
				console.error(err);
				def.reject('Failed to load data: internal error');
			}
		})
		.error (function (data, status, headers, config, statusText) {
			self.cachedWrList = null;
			self.cachedWgId = '';
			def.reject('Failed to load data: ' + statusText);
		});
		
		return def.promise;
	}

	self.getWrDetails = function (wrId, editMode) {
		var def = $q.defer();
		
		var url = ServiceConfig.ServicesURL + '/workrequest/' + wrId;
		if (editMode) {
			url += '?viewMode=true&date=';
		} else {
			url += '?date=';
		}
		url += new Date().getTime();
		
		$http.get(url)
		.success (function (data, status, headers, config, statusText) {
			if (!data || (data && !data.statusCode)) {
				def.reject('Work Request #' + wrId + ' does not exist.');
			}

			if (_isErrorResponse (data)) {
				def.reject(data.statusMessage);
				return;
			}
			
			try {
				// find the current priority record
				for (var i = 0; i < data.priorityList.length; ++i) {
					if (data.priorityId == data.priorityList[i].priorityId) {
						data.priority = data.priorityList[i];
						break;
					}
				}
				
				// find the current status record
				for (var i = 0; i < data.statusList.length; ++i) {
					if (data.statusId == data.statusList[i].statusId) {
						data.status = data.statusList[i];
						break;
					}
				}
		
				// set the current workgroup
				data.assignedGroup = {
					groupId: data.assignedGroupId,
					groupName: data.assignedGroupName
				};
				
				// set the current request type
				data.wrType = {
					wrTypeId: data.wrTypeId,
					wrTypeName: data.wrTypeName
				}

				// convert dates from strings to objects
				for (var i = 0; i < data.wrNotesList.length; ++i) {
					data.wrNotesList[i].createdDate = moment(data.wrNotesList[i].createdDate, 'DD-MMM-YY HH:mm').toDate();
				}
				
				// convert due/etr dates from strings to objects
				if (data.dueDate) {
					data.dueDate = moment(data.dueDate, DATE_FORMAT).toDate();
				}
				if (data.etrDate) {
					data.etrDate = moment(data.etrDate, DATE_FORMAT).toDate();
				}
				
				def.resolve(data);
			} catch (err) {
				console.error(err);
				def.reject('Failed to load data: internal error');
			}
		})
		.error (function (data, status, headers, config, statusText) {
			def.reject('Failed to load data: ' + statusText);
		});
		
		return def.promise;
	}

	self.saveWr = function (wrDetails) {
		var data = {};
		data.lastModifiedTime = wrDetails.lastModifiedTime;
		data.assignedGroupId = wrDetails.assignedGroup.groupId;
		data.statusId = wrDetails.status.statusId;
		if (wrDetails.assignedAnalyst) {
			data.assignedAnalyst = wrDetails.assignedAnalyst;
		}
		data.priorityId = wrDetails.priority.priorityId;
		if (wrDetails.notes) {
			data.notes = wrDetails.notes;
		}
		if (wrDetails.spiInd) {
			data.spiInd = wrDetails.spiInd;
		}
		if (wrDetails.wrType) {
			data.wrTypeId = wrDetails.wrType.wrTypeId;
		}
		if (wrDetails.dueDate) {
			data.dueDate = moment(wrDetails.dueDate).format(DATE_FORMAT);
		}
		if (wrDetails.etrDate) {
			data.etrDate = moment(wrDetails.etrDate).format(DATE_FORMAT);
		}
		
		var def = $q.defer();
		
		$http.post(ServiceConfig.ServicesURL + '/workrequest/' + wrDetails.wrId, data)
		.success (function (data, status, headers, config, statusText) {
			if (data.statusCode && data.statusCode > 200) {
				def.reject({'statusCode': data.statusCode, 'statusMessage': data.statusMessage});
			} else if (data.status && data.status > 200) {
				def.reject({'statusCode': data.status, 'statusMessage': data.message});
			} else {
				def.resolve({'statusMessage': data.statusMessage});
			}
		})
		.error (function (data, status, headers, config, statusText) {
			def.reject({'statusCode': status, 'statusMessage': statusText});
		});
		
		return def.promise;
	}

	self.getWrTypeList = function (wgId) {
		var def = $q.defer();
		
		$http.get(ServiceConfig.ServicesURL + '/workgroups/' + wgId + '/request-types?date=' + new Date().getTime())
		.success (function (data, status, headers, config, statusText) {
			if (_isErrorResponse (data)) {
				def.reject(data.statusMessage);
				return;
			}
			
			try {
				def.resolve(data.listWRTypes);
			} catch (err) {
				console.error(err);
				def.reject('Failed to load data: internal error');
			}
		})
		.error (function (data, status, headers, config, statusText) {
			def.reject('Failed to load data: ' + statusText);
		});
		
		return def.promise;
	}
	
	var _isErrorResponse = function (data) {
		//check for errors from the service
		try {
			if (data.statusCode && data.statusCode != 200) {
				if (!data.statusMessage) {
					data.statusMessage = 'Internal error';
				}
				return true;
			} else if (data.status && data.status != 200) {
				data.statusMessage = data.error;
				return true;
			} else {
				return false;
			}
		} catch (error) {
			data.statusMessage = 'Internal error. ' + error;
			return false;
		}
	}
}); 
