package com.amdocs.cookiemanager;

import java.util.Map;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.json.JSONArray;
import org.json.JSONException;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.util.Log;
import android.webkit.CookieManager;

public class CookiesManager extends CordovaPlugin {
	
	private final String TAG = "CookiesManager";
	
	public static final String PREFS_NAME = "prefs";
	
	private Context mContext;
	
	@Override
	public void initialize(CordovaInterface cordova, CordovaWebView webView) {
	    super.initialize(cordova, webView);
	    
	    Log.d(TAG, "Initializing cookies manager...");
	    
	    mContext = cordova.getActivity();
	    
	    //this will initialize encryption utils and (if needed) create a new key
	    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
	    	EncryptionUtils.init(mContext);
	    } else {
	    	Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
	    }
	}

	@Override
    public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
        if ("clear".equals(action)) {
            return clearAllCookies(callbackContext);
        } else if ("exists".equals(action)) {
        	return isCookieExists(args.getString(0), args.getString(1), callbackContext);
        } else if ("canAutoLogin".equals(action)) {
			return canAutoLogin(args.getString(0), callbackContext);
		} else if ("autoLoginInfoAsCookie".equals(action)) {
			return autoLoginInfoAsCookie(args.getString(0), callbackContext);
		} else if ("isAutoLoginSupported".equals(action)) {
			return isAutoLoginSupported(callbackContext);
		} else if ("storeAutoLoginInfo".equals(action)) {
			return storeAutoLoginInfo(args.getString(0), args.getString(1), args.getString(2), callbackContext);
		} else if ("clearAutoLoginInfo".equals(action)) {
			return clearAutoLoginInfo(args.getString(0), callbackContext);
		} else if ("addAutoLoginCookie".equals(action)) {
			return addAutoLoginCookie(args.getString(0), args.getString(1), callbackContext);
		}
        return false;  // Returning false results in a "MethodNotFound" error.
    }
	
	private boolean clearAllCookies(CallbackContext callbackContext) {
		Log.d(TAG, "Clearing cookies...");
        CookieManager.getInstance().removeAllCookie();
        callbackContext.success();
        return true;
    }
	
	private boolean isCookieExists(String cookieDomain, String cookieName, CallbackContext callbackContext) {
		Log.d(TAG, "Looking for " + cookieName + "@" + cookieDomain + " cookie...");
        String cookiesForDomain = CookieManager.getInstance().getCookie(cookieDomain);
        boolean result = cookiesForDomain.contains(cookieName);
        callbackContext.success(String.valueOf(result));
        return true;
    }
	
	private boolean isAutoLoginSupported(CallbackContext callbackContext) {
		Log.d(TAG, "Checking whether auto-login is supported...");
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
		} else {
			callbackContext.success("true");
		}
        return true;
    }
	
	private boolean storeAutoLoginInfo(String userName, String edsCokieName, String edsCookieDomain, CallbackContext callbackContext) {
		Log.d(TAG, "Storing auto-login information...");
		
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}
		
		// Will only be true if we find the security cookie and able to encrypt and store it
		boolean result = false;
		
		CookieManager cookieMngr = CookieManager.getInstance();
		
		// Extract all cookies for input domain
		String cookiesStr = cookieMngr.getCookie(edsCookieDomain);
		
		if (cookiesStr == null) {
			callbackContext.success("false");
			return true;
		}
		
		// Find the security cookie value
		String[] cookies = cookiesStr.split("; ");
		String edsCookieStr = "";
		for (String cookie: cookies) {
			if (cookie.startsWith(edsCokieName + "=")) {
				edsCookieStr = cookie;
				break;
			}
		}
		
		if (edsCookieStr.isEmpty()) {
			result = false;
		} else {
			//Store the encrypted security cookie string locally
			//and remove it from the browser
			try {
				SharedPreferences.Editor editor = mContext.getSharedPreferences(PREFS_NAME, 0).edit();
				
				//clear previous info and put the new one
				editor.clear().putString(EncryptionUtils.encode(userName), EncryptionUtils.encode(edsCookieStr)).commit();

				//remove the cookie from the browser by setting it's value to empty
				cookieMngr.setCookie(edsCookieDomain, edsCokieName + "=");
				
				//cookie stored locally and removed from the browser; we're good to go!
				result = true;

			} catch (Exception e) {
				Log.d(TAG, e.getMessage());
				result = false;
			}
		}
			
		callbackContext.success(String.valueOf(result));
		return true;
	}
	
	private boolean clearAutoLoginInfo(String userName, CallbackContext callbackContext) {
		Log.d(TAG, "Removing auto-login information...");
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}
		
		SharedPreferences.Editor editor = mContext.getSharedPreferences(PREFS_NAME, 0).edit();
		
		editor.clear().commit();
		
		callbackContext.success("true");
		return true;
	}
	
	private boolean autoLoginInfoAsCookie(String userName, CallbackContext callbackContext) {
		Log.d(TAG, "Retrieving auto-login information...");
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}
		callbackContext.success(getCookieStr(userName));
		return true;
	}

	private String getCookieStr(String userName) {
		// Will be true only if we have auto login information stored locally, input user name matches stored value
		// and we are able to decrypt the values
		String result = "";

		try {
			SharedPreferences prefs = mContext.getSharedPreferences(PREFS_NAME, 0);
			
			//The only key-value pair in this prefs file will be user/cookie; both will be encrypted
			Map<String, ?> prefsValues = prefs.getAll();
			if (prefsValues.size() != 1) {
				result = "";
			} else {
				//Get first (and only) key; which is encrypted user name
				String encryptedUserKey = prefsValues.keySet().toArray(new String[1])[0];
				String userKey = EncryptionUtils.decode(encryptedUserKey);
				
				//Stored user name must be equal to the input user name
				if (!userKey.equals(userName)) {
					result = "";
				} else {
					//Get first (and only) value; which is encrypted <cookie name>=<value> string
					String encryptedCookie = prefsValues.values().toArray(new String[1])[0];
					
					result = EncryptionUtils.decode(encryptedCookie);
				}
			}
		} catch (Exception e) {
			Log.d(TAG, e.getMessage());
			result = "";
		}
		
		return result;
	}
	
	private boolean addAutoLoginCookie(String userName, String edsCookieDomain, CallbackContext callbackContext) {
		Log.d(TAG, "Adding auto-login cookie...");
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}

		// Will be true only if we have auto login infromation stored locally, input user name matches stored value,
		// we are able to decrypt the values and add cookie to browser's cookie store
		boolean result = false;
		
		String cookieStr = getCookieStr(userName);
		
		if (!cookieStr.isEmpty()) {
			CookieManager.getInstance().setCookie(edsCookieDomain, cookieStr);
			result = true;
		}

		callbackContext.success(String.valueOf(result));
		return true;
	}

	private boolean canAutoLogin(String userName, CallbackContext callbackContext) {
		Log.d(TAG, "Looking for auto-login information...");
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}

		// Will be true only if we have auto login infromation stored locally
		// and input user name matches stored value
		boolean result = false;
		
		try {
			SharedPreferences prefs = mContext.getSharedPreferences(PREFS_NAME, 0);
			
			//The only key-value pair in this prefs file will be user/cookie; both will be encrypted
			Map<String, ?> prefsValues = prefs.getAll();
			if (prefsValues.size() != 1) {
				result = false;
			} else {
				//Get first (and only) key; which is encrypted user name
				String encryptedUserKey = prefsValues.keySet().toArray(new String[1])[0];
				String userKey = EncryptionUtils.decode(encryptedUserKey);
				
				//Stored user name must be equal to the input user name
				result = userKey.equals(userName);
			}
		} catch (Exception e) {
			Log.d(TAG, e.getMessage());
			result = false;
		}
		callbackContext.success(String.valueOf(result));
        return true;
	}
}
