package com.amdocs.keymanager;

import java.util.Map;

import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaInterface;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.json.JSONArray;
import org.json.JSONException;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Build;
import android.util.Log;
import android.webkit.CookieManager;

public class KeyManager extends CordovaPlugin {
	
	private final String TAG = "KeyManager";
	
	public static final String PREFS_NAME = "prefs";
	
	private Context mContext;
	
	@Override
	public void initialize(CordovaInterface cordova, CordovaWebView webView) {
	    super.initialize(cordova, webView);

	    Log.d(TAG, "Initializing KeyManager...");

	    mContext = cordova.getActivity();

	    //this will initialize encryption utils and (if needed) create a new key
	    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN_MR2) {
				EncryptionUtils.init(mContext);
	    } else {
	    	Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
	    }
	}

	@Override
	public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
		if ("retrieve".equals(action)) {
			return retrieve(args.getString(0), callbackContext);
		}
		else if ("store".equals(action)) {
			return store(args.getString(0), args.getString(2), callbackContext);
		}
		else if ("remove".equals(action)) {
			return remove(args.getString(0), callbackContext);
		}
		return false;  // Returning false results in a "MethodNotFound" error.
	}

	private boolean retrieve(String key, CallbackContext callbackContext) {
		// Will be true only if information is stored locally, and value is decrypted
		String result = "";
		boolean success = true;

		try {
			SharedPreferences prefs = mContext.getSharedPreferences(PREFS_NAME, 0);

			//The only key-value pair in this prefs file will be key/value; both will be encrypted
			Map<String, ?> prefsValues = prefs.getAll();

			// initial access
			if (prefsValues.size() != 1) {
				success = false;
				result = "Preference file is corrupted, there should be only one entry.";
			}

			else {
				//Get first (and only) key; which is encrypted key
				String storedKey = EncryptionUtils.decode(prefsValues.keySet().toArray(new String[1])[0]);

				//Stored key must be equal to the input
				if (storedKey == null || !storedKey.equals(key)) {
					success = false;
					result = "Stored key does not match given key";
				} else {
					//Get first (and only) value; which is encrypted
					result = EncryptionUtils.decode(prefsValues.values().toArray(new String[1])[0]);
				}
			}
		}
		catch (Exception e) {
			Log.d(TAG, e.getMessage());
			success = false;
			result = e.getMessage();
		}

		if (success) {
			callbackContext.success(result);
		}
		else {
			callbackContext.error(result);
		}

		return true;
	}

	private boolean store(String key, String value, CallbackContext callbackContext) {
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}
		
		// Will only be true if value is encrypted and stored
		try {
			SharedPreferences.Editor editor = mContext.getSharedPreferences(PREFS_NAME, 0).edit();

			//clear previous info and put the new one
			editor.clear().putString(EncryptionUtils.encode(key), EncryptionUtils.encode(value)).commit();

			callbackContext.success();

		} catch (Exception e) {
			Log.d(TAG, e.getMessage());
			callbackContext.error(e.getMessage());
		}

		return true;
	}

	private boolean remove(String key, CallbackContext callbackContext) {
		if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN_MR2) {
			Log.d(TAG, "Android version is below 4.3. Auto-login is not supported");
			callbackContext.success("false");
			return true;
		}
		
		try {
			SharedPreferences.Editor editor = mContext.getSharedPreferences(PREFS_NAME, 0).edit();

			//clear all info
			editor.clear().commit();

			callbackContext.success();

		} catch (Exception e) {
			Log.d(TAG, e.getMessage());
			callbackContext.error(e.getMessage());
		}

		return true;
	}

}